import es, path, zipfile, urllib, base64, time

info = es.AddonInfo()
info.version = '2.5c'
info.author = 'Ojii'
info.thanksto = 'Venjax for the ESAM SDK'

### CHANGELOG ###
#
# 2.5c:
#   -minor bugfix in build
#
# 2.5b:
#   -really fixed it this time
#
# 2.5:
#   -fixed building invalid zipfiles (zipfile module does not add all folders)
#
# 2.4:
#   -added savemode (configuration, see documentation)
#   -turned 'print' into 'dbgmsg'
#   -fixed wrong filename if a info.versionblah line is in the script
#   -last colon on info.version will now turn into build
#
# 2.3a:
#   -fixed error when no buildfile was available and builfileFiles could not be found
#   -fixed error on non-exclude buildfiles to raise an error because bool is not iterable
#
# 2.3:
#   -added easyupload (see documentation)
#
# 2.1:
#   -added output configuration variable for custom output folders (see documentation)
#   -added deloutput and output arguments to buildercfg
#   -fixed bug when no buildfile is found bfp0 syntax is used instead of bfp1 syntax
#   -fixed delextension bug to leave double commas
#
# 2.0:
#   -tested extractDownloadable in IDLE and fixed it. Will work for any 'string' downloadables (= if the path is only string, on variables)
#   -fixed extractDownloadable to break if keywords appear in non-standard context and splitting cannot be done
#   -added configuration file for global username/password and global standard extensions (bfp0 and bfp1)
#   -fixed addinfo not returning on invalid syntax
#   -fixed several issues in bfp1 (searching for bfp0 syntax instead of bfp1 syntax)
#   -fixed normal building routine not to add the last file twice instead of configuration file if one is present
#   -added addcfg method for ingame config editing (see documentation)
#
# 2.0rc3:
#   -added clean() method to prevent any invalid characters in filename
#   -added builfileParser version 1 (bfp1), old buildfiles will be handled by buildfileParser version 0 (bfp0) (see documentation!)
#   -readded the uploadfile_creator (python) to the release
#   -added builder_version public variable
#   NOTE: Using parserversion 1 is highly recommended for any custom builds!
# 
# 2.0rc2:
#   -removed uploadfile_creator from release to get approved
#   -uploadfile_creator is now shipped via the forum thread
#
# 2.0rc1:
#   -first public 2.0 release
#   -added upload capability, added uploadfile
#   -added buildfile for custom builds
#   -added addinfo command for srcds uploadfile creation
#   -added uploadfile_creator for easier creation of uploadfiles


neededkeys = ['username','password','description','esversion','summary','install','loadname','version','releasenotes','fullname']
allkeys = ['username','password','description','esversion','summary','install','loadname','version','releasenotes','fullname','forumurl','publicvariable','featurelistbasename','videourl','ready','status']
singlevars = ['username','password','esversion','summary','loadname','fullname','forumurl','publicvariable','featurelistbasename','videourl','ready','status']
invalid_characters = ['\\', '/', ':' ,'*' ,'?' ,'"' ,'<' ,'>', '|']
gp = path.path(str(es.ServerVar('eventscripts_gamedir')))
cfgfile = gp.joinpath('cfg/builder.cfg')
cfgfile.touch()

def load():
    es.regcmd('build','builder/build') # build basename
    es.regcmd('easyupload','builder/uploadeasy')
    es.regcmd('addinfo','builder/addinfo') # addinfo basename var value
    es.regcmd('buildercfg','builder/addcfg') # buildercfg var value
    es.regcmd('upload','builder/upload') # upload basename
    es.ServerVar('builder_version', info.version).makepublic()
    es.dbgmsg(0, 'BUILDER: Loaded. You\'re using version %s' % info.version)
    es.dbgmsg(0, 'BUILDER: Credits for the ESAM SDK go to Venjax, many thanks!')
    es.dbgmsg(0, 'BUILDER: Please report any bugs or problems on the EventScripts forums')
    es.dbgmsg(0, 'BUILDER: Please submit any feature requests you have to featurelist.org')
    
def clean(version):
    if ':' in version:
        index = version.rfind(':')
        version = version[:index] + '_build' + version[index + 1:]
    for character in invalid_characters:
        version = version.replace(character,'_')
    return version
    
def addinfo():
    if es.getargc() != 4:
        es.dbgmsg(0, 'BUILDER: Invalid syntax! Use: addinfo <basename> <variable> <value>')
        return
    name = es.getargv(1)
    var = es.getargv(2)
    value = es.getargv(3)
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        es.dbgmsg(0, 'BUILDER: Couldn\'t find addon folder!')
        return
    if not var in allkeys:
        es.dbgmsg(0, 'BUILDER: Invalid variable!')
        es.dbgmsg(0, 'BUILDER: Possible variables:')
        es.dbgmsg(0, 'BUILDER: %s' % ','.join(allkeys))
    uf = ap.joinpath('%s.upload' % name)
    L = uf.lines(retain=False)
    done = False
    x = 0
    for line in L:
        if line.startswith(var):
            del L[x]
            L.insert(x,'%s %s' % (var,value))
            done = True
            break
        x += 1
    if not done:
        L.append('%s %s' % (var,value))
    tobedeleted = []
    for line in L:
        ok = False
        for key in allkeys:
            if line.startswith(key):
                ok = True
        if not ok:
            tobedeleted.append(line)
    for line in tobedeleted:
        L.remove(line)
    uf.write_lines(L)
    es.dbgmsg(0, 'BUILDER: Added info: %s %s' % (var,value))
    
def addcfg():
    if es.getargc() != 3:
        es.dbgmsg(0, 'BUILDER: Invalid syntax! Use: buildercfg <variable> <value>')
        return
    var = es.getargv(1)
    value = es.getargv(2)
    allowedvars = ['username','password','extensions','addextension','delextension','output','deloutput']
    if not var in allowedvars:
        es.dbgmsg(0, 'BUILDER: Invalid syntax! Available variables: %s' % ','.join(allowedvars))
        return
    L = cfgfile.lines(retain=False)
    done = False
    if var == 'addextension':
        for line in L:
            if line.startswith('extensions'):
                L[L.index(line)] += ',%s' % value
                done = True
        if not done:
            L.append('extensions %s' % value)
        cfgfile.write_lines(L)
        es.dbgmsg(0, 'BUILDER: Added extension %s to configurations' % value)
    elif var == 'delextension':
        for line in L:
            if line.startswith('extensions'):
                L[L.index(line)] = L[L.index(line)].replace(value,'').replace(',,','')
        cfgfile.write_lines(L)
        es.dbgmsg(0, 'BUILDER: Removed extension %s from configurations' % value)
    elif var == 'deloutput':
        x = None
        for line in L:
            if line.startswith('output'):
                x = L.index(line)
                break
        if x:
            del L[x]
            es.dbgmsg(0, 'BUILDER: Removed output configuration')
        else:
            es.dbgmsg(0, 'BUILDER: Couldn\'t remove output configuration: no output configuration found in configuration file!')
    else:
        for line in L:
            if line.startswith(var):
                L[L.index(line)] += ',%s' % value
                done = True
        if not done:
            L.append('%s %s' % (var,value))
        cfgfile.write_lines(L)
        es.dbgmsg(0, 'BUILDER: Changed %s to %s in configurations' % (var,value)        )

def upload():
    L = cfgfile.lines(retain=False)
    for line in L:
        if line.startswith('savemode'):
            if line.endswith('1') and len(es.getUseridList()) != 0:
                es.dbgmsg(0, 'BUILDER: Cannot upload release!')
                es.dbgmsg(0, 'Savemode is enable and players are on the server!')
                return
            break
    if es.getargc() != 2:
        es.dbgmsg(0, 'BUILDER: Invalid syntax! Use: upload <basename>')
        return
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        es.dbgmsg(0, 'BUILDER: Couldn\'t find addon folder!')
        return
    name = es.getargv(1)
    uf = ap.joinpath('%s.upload' % name)
    if not uf.isfile():
        es.dbgmsg(0, 'BUILDER: Uploadfile not found!')
        es.dbgmsg(0, 'BUILDER: Use addinfo <basename> <var> <value> to add info or add manuall')
        return
    ud = {}
    for line in L:
        if not line.startswith('#'):
            key,val = line.split(' ',1)
            ud[key] = val
    output = None
    if ud.has_key('output'):
        output = path.path(ud['output'])
    L = uf.lines(retain=False)
    for line in L:
        key,val = line.split(' ',1)
        ud[key] = val
    for key in neededkeys:
        if not ud.has_key(key):
            es.dbgmsg(0, 'BUILDER: Not enough information in upload file!')
            es.dbgmsg(0, 'BUILDER: Use addinfo <basename> <var> <value> to add info or add manuall')
            return
    if output:
        zf = output.joinpath('%s_%s.zip' % (name,clean(ud['version'])))
    if not zf.isfile():
        zf = ap.joinpath('%s_%s.zip' % (name,clean(ud['version'])))
    if not zf.isfile():
        es.dbgmsg(0, 'BUILDER: ZIP Archive not found!')
        es.dbgmsg(0, 'BUILDER: Make sure you built the addon before you try to upload!')
        es.dbgmsg(0, 'BUILDER: Use build <basename> to build it')
        return
    ### UPLOADER ###
    ESAMURL = 'http://addons.eventscripts.com/sdk/upload'
    es.dbgmsg(0, 'BUILDER: Starting upload...')
    b64c = base64.b64encode(zf.bytes())
    es.dbgmsg(0, 'BUILDER: Depending on your filesize (in this case: %s Bytes) this might take a while. Grab a beer (or a tea if you\'re a minor)!' % len(b64c))
    raw_params = [
        ('username',ud['username']),
        ('password',ud['password']),
        ('basename',name),
        ('filename',str(zf.basename())),
        ('b64_file_content',b64c),
        ('filesize',zf.getsize()),
        ('file_extension',str(zf.ext)[1:]),
        ('addon_desc',ud['description'].replace('\\n','''
''')),
        ('es_version',ud['esversion']),
        ('addon_summary',ud['summary'][:255]),
        ('install_instructions',ud['install'].replace('\\n','''
''')),
        ('es_load_args',ud['loadname']),
        ('version',ud['version']),
        ('version_notes',ud['releasenotes'].replace('\\n','''
''')),
        ('addon_name',ud['fullname']),
        ('forum_url',ud['forumurl'] if ud.has_key('forumurl') else ''),
        ('public_variable',ud['publicvariable'] if ud.has_key('publicvariable') else ''),
        ('featurelist_url',ud['featurelistbasename'] if ud.has_key('featurlistbasename') else ''),
        ('video_url',ud['videourl'] if ud.has_key('videourl') else ''),
        ('ready4approval',ud['ready'] if ud.has_key('ready') else '1'),
        ('status',ud['status'] if ud.has_key('status') else 'Normal'),
    ]
    params = urllib.urlencode(raw_params)
    upload = urllib.urlopen(ESAMURL,params)
    response = upload.read()
    es.dbgmsg(0, 'BUILDER: Upload finished!')
    es.dbgmsg(0, 'BUILDER: %s' % response)

 ### EXPERIMENTAL! ###
 # Does it work?
 # IDLE tested!
 # Needs a real test...
def extractDownloadable(line):
    if 'es.stringtable' in line: # 2.x
        start = line.find('es.stringtable')
        end = line.find(')',start)
        tline = line[start+15:end] # line between es.stringtable( and )
        if tline.count(',') == 0:
            return False
        table,filepath = tline.split(',')
        table = table.strip().strip('"').strip("'").strip()
        if table == 'downloadables':
            filepath = filepath.strip().strip('"').strip("'").strip()
            download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
            if download.isfile():
                return download
    elif 'es_stringtable' in line: # 1.x
        if line.count(' ') < 2:
            return False
        command, table, filepath = line.split(' ',2)
        if table.strip('"') == 'downloadables':
            filepath = filepath.strip().strip('"').strip("'").strip()
            download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
            if download.isfile():
                return download
    elif 'downloadable' in line: # corelib
        line = line[line.find('downloadable'):]
        if line.count(' ') == 0:
            return False
        command, filepath = line.split(' ',1)
        filepath = filepath.strip().strip('"').strip("'").strip()
        download = gp.joinpath(filepath[1:]) if filepath[0] in ('\\','/') else gp.joinpath(filepath)
        if download.isfile():
            return download
    return False
    
def buildfileParser1(lines, bf,extensions):
    es.dbgmsg(0, 'BUILDER: Parsing buildfile using parser version 1')
    exclude = []
    strict = False
    buildfileFiles = []
    for line in lines: # scan for strict mode and extensions
        if line == 'strict':
            strict = True
            es.dbgmsg(0, 'BUILDER: strict mode enabled')
        elif line.startswith('extensions'):
            extensions = line[10:].split(',')
    for line in lines: # do a new loop because of the extensions
        if not line.startswith('#'): # comments...
            if line[0:2] == '++':
                line = line[2:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    buildfileFiles.append(f)
                elif f.isdir():
                    for ext in extensions:
                        ex = ext.strip()
                        for ff in f.walkfiles('*.%s' % ex):
                            buildfileFiles.append(ff)
            elif line[0] == '+':
                line = line[1:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    buildfileFiles.append(f)
                elif f.isdir():
                    for ext in extensions:
                        ex = ext.strip()
                        for ff in f.files('*.%s' % ex):
                            buildfileFiles.append(ff)
            elif line[0] == '-':
                line = line[1:].strip()
                f = gp.joinpath(line[1:]) if line[0] in ('\\','/') else gp.joinpath(line)
                if f.isfile():
                    exclude.append(f.normpath())
                elif f.isdir():
                    for ff in f.walkfiles():
                        exclude.append(ff.normpath())
    return strict, exclude, extensions, buildfileFiles

def buildfileParser0(lines, bf,extensions):
    es.dbgmsg(0, 'BUILDER: Parsing buildfile using parser version 0')
    exclude = []
    buildfileFiles = []
    strict = False
    firstline = lines[0] if len(lines) > 0 else ''
    secondline = lines[1] if len(lines) > 1 else ''
    if firstline.startswith('__extensions__'):
        extensions = firstline[14:].split(',')
        es.dbgmsg(0, 'BUILDER: Extensions: %s' % ','.join(extensions))
    else:
        lines.insert(0,'__extensions__ %s' % ','.join(extensions))
        bf.write_lines(lines)
    if firstline == '__strict__':
        es.dbgmsg(0, 'BUILDER: Strict build mode enabled')
        strict = True
    if secondline == '__strict__':
        es.dbgmsg(0, 'BUILDER: Strict build mode enabled')
        strict = True
    elif firstline == '__exclude__' or secondline == '__exclude__':
        exclude = []
        es.dbgmsg(0, 'BUILDER: Exclude mode enabled')
        for line in lines:
            if not line.startswith('__'):
                f = gp.joinpath(line[1:] if line[0] in ('/','\\') else line)
                if f.isfile():
                    exclude.append(f.normpath())
                elif f.isdir():
                    for ff in f.walkfiles():
                        exclude.append(ff.normpath())
    for line in lines:
        if not line.startswith('__'):
            f = gp.joinpath(line[1:] if line[0] in ('/','\\') else line)
            if f.isfile():
                buildfileFiles.append(f)
            elif f.isdir():
                for ff in f.walkfiles():
                    buildfileFiles.append(ff)
    return strict, exclude, extensions, buildfileFiles
    
def allfolders(fldr):
    current = ''
    fldr = fldr.replace('\\','/')
    folders = fldr.split('/')
    folders.pop(-1)
    allfolders = []
    for x in folders:
        current = '%s%s/' % (current,x)
        allfolders.append(current)
    return allfolders
    
def addFolder(z,folder):
    zinfo = zipfile.ZipInfo(folder,time.localtime()[:6])
    zinfo.external_attr = 48
    z.writestr(zinfo, "")
 
def build():
    es.dbgmsg(0, '')
    if es.getargc() != 2:
        es.dbgmsg(0, 'BUILDER: Invalid syntax! Use: build <basename>')
        return
    ap = path.path(es.getAddonPath(es.getargv(1)))
    if not ap.isdir():
        es.dbgmsg(0, 'BUILDER: Couldn\'t find addon: %s' % es.getargv(1))
        return
    name = es.getargv(1)
    es.dbgmsg(0, 'BUILDER: Scanning main file for version info and downloadables')
    version = '1'
    main = ap.joinpath('%s.py' % name) if ap.joinpath('%s.py' % name).isfile() else ap.joinpath('es_%s.txt' % name)
    if not main.isfile():
        es.dbgmsg(0, 'BUILDER: Couldn\'t find main script file!')
        return
    uf = ap.joinpath('%s.upload' % name)
    L = []
    if uf.isfile():
        L = uf.lines(retain=False)
    downloadables = []
    done = []
    for line in main.lines(retain=False):
        cline = line.strip()
        if cline.startswith('info.version ') or cline.startswith('info.version\t') or cline.startswith('info.version='):
            version = line.split('=')[1].strip().strip("'").strip('"')
            x = 0
            versiondone = False
            for line in L:
                if line.startswith('version'):
                    del L[x]
                    L.insert(x,'version %s' % version)
                    versiondone = True
                    break
                x += 1
            if not versiondone:
                L.append('version %s' % version)
            es.dbgmsg(0, 'BUILDER: Version found: %s' % version)
        if 'stringtable' in line and 'downloadables' in line:
            tmp = extractDownloadable(line)
            if tmp:
                es.dbgmsg(0, 'BUILDER: Found downloadable %s' % tmp)
                downloadables.append(tmp)
        elif 'downloadable' in line:
            tmp = extractDownloadable(line)
            if tmp:
                es.dbgmsg(0, 'BUILDER: Found downloadable %s' % tmp)
                downloadables.append(tmp)
    es.dbgmsg(0, 'BUILDER: Writing zipfile')
    bf = ap.joinpath('%s.build' % name)
    extensions = None
    output = ap
    L2 = cfgfile.lines(retain=False)
    for line in L2:
        if not line.startswith('#'):
            if line.startswith('extensions'):
                command, extensions = line.split(' ',1)
                extensions = extensions.split(',')
            elif line.startswith('output'):
                command, output = line.split(' ',1)
                output = path.path(output)
                es.dbgmsg(0, 'BUILDRE: Output path found %s' % output)
                if not output.isdir():
                    output.mkdir()
    if not extensions:
        extensions = ['py','txt','ini','cfg','res']
    strict = False
    exclude = []
    buildfileFiles = []
    if bf.isfile():
        es.dbgmsg(0, 'BUILDER: Buildfile found')
        bflines = bf.lines(retain=False)
        version1 = False
        for line in bflines:
            if line.startswith('parserversion 1'):
                version1 = True
                strict, exclude, extensions, buildfileFiles = buildfileParser1(bflines, bf, extensions)
                break
        if not version1:
            strict, exclude, extensions, buildfileFiles = buildfileParser0(bflines, bf, extensions)
    else:
        bf.write_text('parserversion 1\nextensions %s' % ','.join(extensions))
    zf = output.joinpath('%s_%s.zip' % (name,clean(version)))
    zip = zipfile.ZipFile(zf,'w')
    folders = []
    if not strict:
        for ext in extensions:
            ex = '.%s' % ext.strip()
            for f in ap.walkfiles('*%s' % ex):
                if f.normpath() not in exclude:
                    currentfiles = allfolders(gp.relpathto(f))
                    for cf in currentfiles:
                        if not cf in folders:
                            folders.append(cf)
                            es.dbgmsg(0, 'BUILDER: Adding %s' % cf.encode('ascii'))
                            addFolder(zip,cf.encode('ascii'))
                    es.dbgmsg(0, 'BUILDER: Adding %s' % f.encode('ascii'))
                    zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
                    done.append(f)
        for downloadable in downloadables:
            if downloadable.normpath() not in exclude:
                    currentfiles = allfolders(gp.relpathto(downloadable))
                    for cf in currentfiles:
                        if not cf in folders:
                            folders.append(cf)
                            es.dbgmsg(0, 'BUILDER: Adding %s' % cf.encode('ascii'))
                            addFolder(zip,cf.encode('ascii'))
                    es.dbgmsg(0, 'BUILDER: Adding %s' % downloadable.encode('ascii'))
                    zip.write(downloadable.encode('ascii'),gp.relpathto(downloadable).encode('ascii'))
                    done.append(downloadable)
        f = gp.joinpath('cfg/%s.cfg' % name)
        if f.isfile():
            if f.normpath() not in exclude:
                currentfiles = allfolders(gp.relpathto(f))
                for cf in currentfiles:
                    if not cf in folders:
                        folders.append(cf)
                        es.dbgmsg(0, 'BUILDER: Adding %s' % cf.encode('ascii'))
                        addFolder(zip,cf.encode('ascii'))
                es.dbgmsg(0, 'BUILDER: Adding %s' % f.encode('ascii'))
                zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
                done.append(f)
    for f in buildfileFiles:
        if f not in done and f not in exclude:
            currentfiles = allfolders(gp.relpathto(f))
            for cf in currentfiles:
                if not cf in folders:
                    folders.append(cf)
                    es.dbgmsg(0, 'BUILDER: Adding %s' % cf.encode('ascii'))
                    addFolder(zip,cf.encode('ascii'))
            es.dbgmsg(0, 'BUILDER: Adding %s' % f.encode('ascii'))
            zip.write(f.encode('ascii'),gp.relpathto(f).encode('ascii'))
            done.append(f)
    zip.close()
    uf.write_lines(L)
    es.dbgmsg(0, 'BUILDER: Built addon in file %s' % zf)
    es.dbgmsg(0, 'BUILDER: Done')
    es.dbgmsg(0, '')

def uploadeasy():
    L = cfgfile.lines(retain=False)
    for line in L:
        if line.startswith('savemode'):
            if line.endswith('1') and len(es.getUseridList()) != 0:
                es.dbgmsg(0, 'BUILDER: Cannot upload release!')
                es.dbgmsg(0, 'Savemode is enable and players are on the server!')
                return
            break
    es.dbgmsg(0, 'BUILDER: Easyupload')
    es.dbgmsg(0, 'BUILDER: If values are given in brackets, just leave the line empty to keep them')
    es.dbgmsg(0, 'BUILDER: If a multiline input is asked, use END to finish the input')
    basename = raw_input('Basename:\n')
    if path.path(es.getAddonPath(basename)).isdir():
        easyuploader.start(basename)
    else:
        es.dbgmsg(0, 'BUILDER: Couldn\'t find addon folder!')
    
class easyupload:
    def __init__(self):
        self.positions = ['username','password','description','esversion','summary','install','loadname','releasenotes','fullname','forumurl','publicvariable','featurelistbasename','videourl','ready','status']
        self.currentposition = self.positions[0]
        
    def start(self,basename):
        self.basename = basename
        ap = path.path(es.getAddonPath(basename))
        self.uf = ap.joinpath('%s.upload' % basename)
        self.oldvalues = {}
        self.values = {}
        if not self.uf.isfile:
            es.dbgmsg(0, 'BUILDER: Build the release first! No uploadfile found!')
            return
        output = False
        L = cfgfile.lines(retain=False)
        for line in L:
            if line.startswith('output') and line.count(' ') != 0:
                var,val = line.split(' ',1)
                output = path.path(val)
            elif line.startswith('username') or line.startswith('password') and line.count(' ') != 0:
                var,val = line.split(' ',1)
                self.oldvalues[var] = val
        for line in self.uf.lines(retain=False):
            if line.count(' ') != 0:
                var,val = line.split(' ',1)
                self.oldvalues[var] = val
        self.values['version'] = self.oldvalues['version']
        self.zip = False
        if output:
            self.zip = output.joinpath('%s_%s.zip' % (basename,clean(self.values['version'])))
            es.dbgmsg(0, self.zip)
        if not self.zip or not self.zip.isfile():
            self.zip = ap.joinpath('%s_%s.zip' % (basename,clean(self.values['version'])))
        if not self.zip.isfile():
            es.dbgmsg(0, self.zip)
            es.dbgmsg(0, 'BUILDER: Zipfile not found!')
            return
        self.getcurrent()

    def getcurrent(self):
        if self.currentposition in singlevars:
            question = self.currentposition if not self.oldvalues.has_key(self.currentposition) else '%s (%s)' % (self.currentposition, self.oldvalues[self.currentposition])
            current = raw_input('%s:\n' % question)
            if current == '' and self.oldvalues.has_key(self.currentposition):
                current = self.oldvalues[self.currentposition]
            self.values[self.currentposition] = current
            self.next()
        else:
            question = self.currentposition if not self.oldvalues.has_key(self.currentposition) else '%s (%s)' % (self.currentposition, self.oldvalues[self.currentposition])
            es.dbgmsg(0, '%s:' % question.replace('\\n','\n'))
            acceptoldval = True if self.oldvalues.has_key(self.currentposition) or self.currentposition not in neededkeys else False
            self.values[self.currentposition] = self.getmultiline(acceptoldval)
            self.next()
            
    def getmultiline(self,acceptoldval):
        multi = []
        while True:
            part = raw_input('')
            if part == '' and len(multi) == 0 and acceptoldval:
                return self.oldvalues[self.currentposition]
            elif part == 'END':
                return '\n'.join(multi)
            else:
                multi.append(part)
        
    def next(self):
        if self.currentposition == self.positions[len(self.positions) - 1]:
            self.finish()
        else:
            self.currentposition = self.positions[self.positions.index(self.currentposition) + 1]
            self.getcurrent()
            
    def finish(self):
        ESAMURL = 'http://addons.eventscripts.com/sdk/upload'
        es.dbgmsg(0, 'BUILDER: Starting upload...')
        b64c = base64.b64encode(self.zip.bytes())
        es.dbgmsg(0, 'BUILDER: Depending on your filesize (in this case: %s Bytes) this might take a while. Grab a beer (or a tea if you\'re a minor)!' % len(b64c))
        raw_params = [
            ('username',self.values['username']),
            ('password',self.values['password']),
            ('basename',self.basename),
            ('filename',str(self.zip.basename())),
            ('b64_file_content',b64c),
            ('filesize',self.zip.getsize()),
            ('file_extension',str(self.zip.ext)[1:]),
            ('addon_desc',self.values['description'].replace('\\n','''
''')),
            ('es_version',self.values['esversion']),
            ('addon_summary',self.values['summary'][:255]),
            ('install_instructions',self.values['install'].replace('\\n','''
''')),
            ('es_load_args',self.values['loadname']),
            ('version',self.values['version']),
            ('version_notes',self.values['releasenotes'].replace('\\n','''
''')),
            ('addon_name',self.values['fullname']),
            ('forum_url',self.values['forumurl']),
            ('public_variable',self.values['publicvariable']),
            ('featurelist_url',self.values['featurelistbasename']),
            ('video_url',self.values['videourl']),
            ('ready4approval',self.values['ready']),
            ('status',self.values['status']),
        ]
        params = urllib.urlencode(raw_params)
        upload = urllib.urlopen(ESAMURL,params)
        response = upload.read()
        es.dbgmsg(0, 'BUILDER: Upload finished!')
        es.dbgmsg(0, 'BUILDER: %s' % response)
        self.currentposition = self.positions[0]
        L = []
        for var in self.values:
            L.append('%s %s' % (var,self.values[var]))
        self.uf.write_lines(L)
        es.dbgmsg(0, 'BUILDER: Uploadfile written')
        
easyuploader = easyupload()
